﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;

namespace NLScC.model
{
    internal class LogConsole : System.Windows.Controls.RichTextBox, INotifyPropertyChanged
    {
        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        private void  NotifyPropertyChanged([CallerMemberName] string PropertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(PropertyName));
        }

        #endregion

        #region @override

        protected override void OnMouseEnter(MouseEventArgs e)
        {
            this.Cursor = Cursors.Arrow;
            base.OnMouseEnter(e);
        }

        #endregion

        #region public Members

        private int _errorCount;
        public int ErrorCount
        {
            get { return _errorCount; }
            set { 
                _errorCount = value;
                NotifyPropertyChanged();
                }
        }

        private int _warningCount;
        public int WarningCount
        {
            get { return _warningCount; }
            set
            {
                _warningCount = value;
                NotifyPropertyChanged();
            }
        }

        private int _infoCount;
        public int InfoCount
        {
            get { return _infoCount; }
            set
            {
                _infoCount = value;
                NotifyPropertyChanged();
            }
        }

        private LogLevel _errorLevel;
        public LogLevel ErrorLevel
        {
            get { return _errorLevel; }
            set
            {
                if (_errorLevel != value)
                {
                    _errorLevel = value;
                    NotifyPropertyChanged();
                }
            }
        }
        #endregion

        public struct Errors
        {
            public Errors(string message, LogLevel level)
            {
                Message = message;
                Level = level;
            }
            public string Message { get; }
            public LogLevel Level { get; }
        }

        private static List<Errors> _errorList { get; set; }
        public static List<Errors> ErrorList
        {
            get
            {
                if (_errorList == null) _errorList = new List<Errors>();
                return _errorList;
            }
        }

        public static void Log(string message, LogLevel level = LogLevel.Detail)
        {
            if (LogConsole.Logger != null)
            {
                LogConsole.Logger.LogInternal(message, level);
            }
            else
            {
                LogConsole.ErrorList.Add(new LogConsole.Errors(message, level));
            }
        }


        private static LogConsole _logger { get; set; }
        public static LogConsole Logger
        {
            get {return _logger; }
        }


        public LogConsole()
        {
            this.Document.Blocks.Clear();
            this.Background = (System.Windows.Media.Brush)new System.Windows.Media.BrushConverter().ConvertFromString("gray");
            this.FontFamily = new System.Windows.Media.FontFamily("Courier New");
            ResetLogLevel();
            FontWeight = System.Windows.FontWeights.ExtraBold;
            _logger = this;
            this.LogInternal("Program started.");
            ReadErrorList();
        }

        private void ReadErrorList()
        {
            foreach(Errors e in ErrorList)
            {
                this.LogInternal(e.Message, e.Level);
            }
        }


        public enum LogLevel
        {
            Error = 1,
            Warning = 2,
            Info = 4,
            Detail = 8
        }

        public void LogInternal(string Message, LogLevel level = LogLevel.Detail)
        {
            System.Windows.Media.Brush brush = BrushFromLogLevel(level);
            this.Document.Blocks.Add(new Paragraph(new Run(LogLevelPrefix(level) + DateTime.Now + " - " + Message) { Foreground = brush }));
        }

        private System.Windows.Media.Brush BrushFromLogLevel(LogLevel level)
        {
            switch (level)
            {
                case LogLevel.Error:
                    return System.Windows.Media.Brushes.DarkRed;
                case LogLevel.Warning:
                    return System.Windows.Media.Brushes.DarkOrange;
                default:
                    return System.Windows.Media.Brushes.Black;
            }
        }

        private string LogLevelPrefix(LogLevel level)
        {
            switch (level)
            {
                case LogLevel.Error:
                    ErrorCount++;
                    ErrorLevel = LogLevel.Error;
                    return "Error   : ";
                case LogLevel.Warning:
                    if (ErrorLevel != LogLevel.Error) ErrorLevel = LogLevel.Warning;
                    WarningCount++;
                    return "Warning :";
                case LogLevel.Info:
                    InfoCount++;
                    return "Info    :";
                default:
                    return "Detailed:";
            }
        }

        public void ResetLogLevel()
        {
            ErrorLevel = LogLevel.Detail;
        }

    }
}
